"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthService = void 0;
const common_1 = require("@nestjs/common");
const prisma_service_1 = require("../prisma/prisma.service");
const argon = require("argon2");
const client_1 = require("@prisma/client");
const jwt_1 = require("@nestjs/jwt");
const config_1 = require("@nestjs/config");
let AuthService = class AuthService {
    constructor(prisma, jwt, config) {
        this.prisma = prisma;
        this.jwt = jwt;
        this.config = config;
    }
    async signup(dto) {
        const hash = await argon.hash(dto.password);
        const promoCode = await this.prisma.promoCode.findFirst({
            where: {
                name: dto.promoCode,
            },
        });
        const userRole = await this.prisma.role.findFirst({
            where: {
                name: 'user',
            },
        });
        let balance = 1000;
        if (promoCode && promoCode.value) {
            balance += promoCode.value;
        }
        try {
            const user = await this.prisma.user.create({
                data: {
                    firstName: dto.firstName,
                    lastName: dto.lastName,
                    pseudo: dto.pseudo,
                    city: dto.city,
                    email: dto.email,
                    hash,
                    age: dto.age,
                    roleId: userRole.id,
                    isActive: true,
                    dollarAvailables: balance,
                },
            });
            return this.signToken(user);
        }
        catch (error) {
            if (error instanceof client_1.Prisma.PrismaClientKnownRequestError) {
                if (error.code === 'P2002') {
                    throw new common_1.ForbiddenException('Credentials taken');
                }
            }
            throw error;
        }
    }
    async signin(dto) {
        const userDatas = await this.prisma.user.findUnique({
            where: {
                email: dto.email,
            },
            include: {
                UserHasCrypto: {
                    include: { Crypto: true },
                },
                Role: true,
            },
        });
        if (!userDatas)
            throw new common_1.ForbiddenException('Credentials incorrect');
        const pwMatches = await argon.verify(userDatas.hash, dto.password);
        if (!pwMatches)
            throw new common_1.ForbiddenException('Credentials incorrect');
        return this.signToken(userDatas);
    }
    async signToken(user) {
        const payload = {
            sub: user.id,
            email: user.email,
        };
        user.hash = null;
        const secret = this.config.get('JWT_SECRET');
        const token = await this.jwt.signAsync(payload, {
            expiresIn: '30d',
            secret: secret,
        });
        return {
            access_token: token,
            user,
        };
    }
};
exports.AuthService = AuthService;
exports.AuthService = AuthService = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [prisma_service_1.PrismaService,
        jwt_1.JwtService,
        config_1.ConfigService])
], AuthService);
//# sourceMappingURL=auth.service.js.map