"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CryptoService = void 0;
const common_1 = require("@nestjs/common");
const prisma_service_1 = require("../prisma/prisma.service");
const checkUser_1 = require("../utils/checkUser");
let CryptoService = class CryptoService {
    constructor(prisma) {
        this.prisma = prisma;
    }
    async getCryptos(userId) {
        await (0, checkUser_1.checkUserHasAccount)(userId);
        return this.prisma.crypto.findMany({
            orderBy: {
                name: 'asc',
            },
        });
    }
    async searchCryptos(userId, cryptoName) {
        await (0, checkUser_1.checkUserHasAccount)(userId);
        return this.prisma.crypto.findMany({
            where: {
                name: {
                    contains: cryptoName,
                    mode: 'insensitive',
                },
            },
            orderBy: {
                name: 'asc',
            },
        });
    }
    async getCryptoHistory(userId, cryptoId) {
        await (0, checkUser_1.checkUserHasAccount)(userId);
        if (cryptoId) {
            return this.prisma.cryptoHistory.findMany({
                where: {
                    id_crypto: cryptoId,
                },
                orderBy: {
                    created_at: 'desc',
                },
                take: 50,
            });
        }
        else {
            throw new common_1.ForbiddenException('Crypto UUID required');
        }
    }
    async sellCryto(userId, dto) {
        await (0, checkUser_1.checkUserHasAccount)(userId);
        const crypto = await this.prisma.crypto.findFirst({
            where: {
                id: dto.id_crypto,
            },
        });
        const user = await this.prisma.user.findFirst({
            where: { id: userId },
        });
        if (!crypto || !crypto.id) {
            throw new common_1.ForbiddenException('Crypto doesnt exist');
        }
        const userAsset = await this.prisma.userHasCrypto.findFirst({
            where: {
                id_user: userId,
                id_crypto: dto.id_crypto,
            },
        });
        const newCryptoValue = crypto.value * 0.9;
        if (!userAsset || userAsset.amount < dto.amount) {
            throw new common_1.ForbiddenException(`Seller dont have enough asset`);
        }
        else {
            await this.prisma.crypto.update({
                where: {
                    id: crypto.id,
                },
                data: {
                    value: newCryptoValue,
                },
            });
            const price = crypto.value * dto.amount;
            const newBalanceSeller = user.dollarAvailables + price;
            await this.prisma.user.update({
                where: {
                    id: userId,
                },
                data: {
                    dollarAvailables: newBalanceSeller,
                },
            });
            await this.prisma.cryptoHistory.create({
                data: {
                    id_crypto: crypto.id,
                    value: newCryptoValue,
                },
            });
            const newBalance = userAsset.amount - dto.amount;
            await this.prisma.crypto.update({
                where: {
                    id: dto.id_crypto,
                },
                data: {
                    quantity: crypto.quantity + dto.amount,
                },
            });
            if (newBalance > 0) {
                return this.prisma.userHasCrypto.update({
                    where: {
                        id: userAsset.id,
                    },
                    data: {
                        amount: newBalance,
                    },
                });
            }
            else {
                return this.prisma.userHasCrypto.delete({
                    where: {
                        id: userAsset.id,
                    },
                });
            }
        }
    }
    async createCrypto(userId, dto) {
        await (0, checkUser_1.checkuserIsAdmin)(userId);
        const existingCryptosWithSameName = await this.prisma.crypto.findMany({
            where: {
                name: dto.name,
            },
        });
        if (existingCryptosWithSameName.length > 0) {
            throw new common_1.ForbiddenException('Name already taken');
        }
        const crypto = await this.prisma.crypto.create({
            data: {
                name: dto.name,
                image: dto.image,
                value: dto.value,
                quantity: dto.quantity,
            },
        });
        return crypto;
    }
    async buyCrypto(userId, dto) {
        const crypto = await this.prisma.crypto.findFirst({
            where: {
                id: dto.id_crypto,
            },
        });
        const user = await this.prisma.user.findFirst({
            where: {
                id: userId,
            },
        });
        if (crypto.quantity < dto.amount) {
            throw new common_1.ForbiddenException('No more tokens available');
        }
        const necessaryAmount = crypto.value * dto.amount;
        console.log(necessaryAmount, user.dollarAvailables);
        if (necessaryAmount > user.dollarAvailables) {
            throw new common_1.ForbiddenException('Make money first :) ');
        }
        else {
            const userAsset = await this.prisma.userHasCrypto.findFirst({
                where: {
                    id_crypto: dto.id_crypto,
                    id_user: userId,
                },
            });
            const newBalance = user.dollarAvailables - necessaryAmount;
            console.log(newBalance);
            await this.prisma.user.update({
                where: {
                    id: user.id,
                },
                data: {
                    dollarAvailables: newBalance,
                },
            });
            if (userAsset) {
                const newBalance = userAsset.amount + dto.amount;
                await this.prisma.userHasCrypto.update({
                    where: {
                        id: userAsset.id,
                    },
                    data: {
                        amount: newBalance,
                    },
                });
            }
            else {
                await this.prisma.userHasCrypto.create({
                    data: {
                        id_crypto: dto.id_crypto,
                        id_user: userId,
                        amount: dto.amount,
                    },
                });
            }
        }
        const newCryptoValue = crypto.value * 1.1;
        await this.prisma.cryptoHistory.create({
            data: {
                id_crypto: crypto.id,
                value: newCryptoValue,
            },
        });
        const newQuantity = (crypto.quantity -= dto.amount);
        return this.prisma.crypto.update({
            where: {
                id: dto.id_crypto,
            },
            data: {
                value: newCryptoValue,
                quantity: newQuantity,
            },
        });
    }
    async editCryptoById(userId, cryptoId, dto) {
        await (0, checkUser_1.checkuserIsAdmin)(userId);
        const crypto = await this.prisma.crypto.findUnique({
            where: {
                id: cryptoId,
            },
        });
        if (!crypto || crypto.id !== cryptoId)
            throw new common_1.ForbiddenException('Access to resources denied');
        return this.prisma.crypto.update({
            where: {
                id: crypto.id,
            },
            data: {
                ...dto,
            },
        });
    }
    async deleteCryptoById(userId, id) {
        await (0, checkUser_1.checkuserIsAdmin)(userId);
        const crypto = await this.prisma.crypto.findUnique({
            where: {
                id: id,
            },
        });
        if (!crypto || crypto.id !== id)
            throw new common_1.ForbiddenException('Access to resources denied');
        await this.prisma.crypto.delete({
            where: {
                id: crypto.id,
            },
        });
    }
};
exports.CryptoService = CryptoService;
exports.CryptoService = CryptoService = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [prisma_service_1.PrismaService])
], CryptoService);
//# sourceMappingURL=crypto.service.js.map