"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OfferService = void 0;
const common_1 = require("@nestjs/common");
const prisma_service_1 = require("../prisma/prisma.service");
const checkUser_1 = require("../utils/checkUser");
let OfferService = class OfferService {
    constructor(prisma) {
        this.prisma = prisma;
    }
    async getOffers(userId) {
        await (0, checkUser_1.checkUserHasAccount)(userId);
        return this.prisma.offer.findMany({
            orderBy: {
                created_at: 'desc',
            },
            select: {
                id: true,
                User: {
                    select: {
                        pseudo: true,
                    },
                },
                amount: true,
                created_at: true,
                id_user: true,
                Crypto: true,
            },
        });
    }
    async getOffersByCrypto(userId, cryptoId) {
        await (0, checkUser_1.checkUserHasAccount)(userId);
        return this.prisma.offer.findMany({
            where: {
                id_crypto: cryptoId,
            },
            orderBy: {
                amount: 'desc',
            },
            select: {
                id: true,
                User: {
                    select: {
                        pseudo: true,
                    },
                },
                amount: true,
                created_at: true,
                id_user: true,
                Crypto: true,
            },
        });
    }
    async createOffer(userId, dto) {
        await (0, checkUser_1.checkUserHasAccount)(userId);
        const userAssets = await this.prisma.userHasCrypto.findFirst({
            where: {
                id_user: userId,
                Crypto: {
                    id: dto.id_crypto,
                },
            },
        });
        if (!userAssets) {
            throw new common_1.ForbiddenException('No tokens available');
        }
        if (userAssets.amount < dto.amount) {
            throw new common_1.ForbiddenException('Insuficient tokens avaiblable');
        }
        const currentUserOffers = await this.prisma.offer.findMany({
            where: {
                id_user: userId,
                id_crypto: dto.id_crypto,
            },
        });
        let totalAmountsInOffers = 0;
        currentUserOffers.forEach((offer) => {
            totalAmountsInOffers += offer.amount;
        });
        console.log(totalAmountsInOffers);
        if (totalAmountsInOffers >= userAssets.amount) {
            throw new common_1.ImATeapotException('Insuficient tokens avaiblable');
        }
        const offer = await this.prisma.offer.create({
            data: {
                id_crypto: dto.id_crypto,
                id_user: userId,
                amount: dto.amount,
            },
        });
        return offer;
    }
    async editOfferById(userId, offerId, dto) {
        await (0, checkUser_1.checkUserHasAccount)(userId);
        const offer = await this.prisma.offer.findUnique({
            where: {
                id: offerId,
            },
        });
        const crypto = await this.prisma.crypto.findUnique({
            where: {
                id: dto.id_crypto,
            },
        });
        if (!crypto || !crypto.id) {
            throw new common_1.ForbiddenException('Crypto doesnt exist');
        }
        if (!offer || offer.id !== offerId)
            throw new common_1.ForbiddenException('Offer id mandatory');
        return this.prisma.offer.update({
            where: {
                id: offerId,
            },
            data: {
                ...dto,
            },
        });
    }
    async deleteOfferById(userId, id) {
        await (0, checkUser_1.checkuserIsAdmin)(userId);
        const offer = await this.prisma.offer.findUnique({
            where: {
                id: id,
            },
        });
        if (!offer || offer.id !== id)
            throw new common_1.ForbiddenException('Access to resources denied');
        await this.prisma.offer.delete({
            where: {
                id: id,
            },
        });
    }
};
exports.OfferService = OfferService;
exports.OfferService = OfferService = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [prisma_service_1.PrismaService])
], OfferService);
//# sourceMappingURL=offer.service.js.map