"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TradeService = void 0;
const common_1 = require("@nestjs/common");
const prisma_service_1 = require("../prisma/prisma.service");
const checkUser_1 = require("../utils/checkUser");
let TradeService = class TradeService {
    constructor(prisma) {
        this.prisma = prisma;
    }
    async getTrades(userId) {
        await (0, checkUser_1.checkuserIsAdmin)(userId);
        return this.prisma.trade.findMany({
            orderBy: {
                created_at: 'desc',
            },
            select: {
                Giver: {
                    select: {
                        firstName: true,
                        lastName: true,
                        pseudo: true,
                        dollarAvailables: true,
                    },
                },
                Receiver: {
                    select: {
                        firstName: true,
                        lastName: true,
                        pseudo: true,
                        dollarAvailables: true,
                    },
                },
                Crypto: true,
                id: true,
            },
        });
    }
    async getLastTrades() {
        return this.prisma.trade.findMany({
            orderBy: {
                created_at: 'desc',
            },
            select: {
                amount_traded: true,
                Crypto: true,
            },
        });
    }
    async createTrade(userId, dto) {
        await (0, checkUser_1.checkUserHasAccount)(userId);
        const offer = await this.prisma.offer.findUnique({
            where: {
                id: dto.id_offer,
            },
        });
        const crypto = await this.prisma.crypto.findFirst({
            where: {
                id: offer.id_crypto,
            },
        });
        const buyer = await this.prisma.user.findFirst({
            where: {
                id: userId,
            },
        });
        const seller = await this.prisma.user.findFirst({
            where: {
                id: offer.id_user,
            },
        });
        if (seller.id === buyer.id) {
            throw new common_1.ImATeapotException('Get the fuck outta here');
        }
        const price = crypto.value * offer.amount;
        if (buyer.dollarAvailables < price) {
            throw new common_1.ForbiddenException(`Acqueror ${buyer.pseudo} doesnt have enough money to make this trade`);
        }
        const asset = await this.prisma.userHasCrypto.findFirst({
            where: {
                id_crypto: offer.id_crypto,
                id_user: offer.id_user,
            },
        });
        if (!asset || asset.amount < offer.amount) {
            throw new common_1.ForbiddenException(`Seller doesnt have enough ${crypto.name} `);
        }
        const trade = await this.prisma.trade.create({
            data: {
                id_giver: offer.id_user,
                id_receiver: userId,
                id_crypto: offer.id_crypto,
                amount_traded: offer.amount,
            },
        });
        const newBalanceGiver = (asset.amount -= offer.amount);
        await this.prisma.userHasCrypto.update({
            where: {
                id: asset.id,
            },
            data: {
                amount: newBalanceGiver,
            },
        });
        const receiverAssets = await this.prisma.userHasCrypto.findFirst({
            where: {
                id_user: userId,
                id_crypto: offer.id_crypto,
            },
        });
        if (!receiverAssets) {
            await this.prisma.userHasCrypto.create({
                data: {
                    id_user: userId,
                    id_crypto: offer.id_crypto,
                    amount: offer.amount,
                    createdAt: new Date(),
                },
            });
        }
        else {
            const newBalanceReceiver = receiverAssets.amount + offer.amount;
            await this.prisma.userHasCrypto.update({
                where: {
                    id: receiverAssets.id,
                },
                data: {
                    amount: newBalanceReceiver,
                },
            });
        }
        const newValue = crypto.value * 1.1;
        await this.prisma.cryptoHistory.create({
            data: {
                id_crypto: crypto.id,
                value: newValue,
            },
        });
        await this.prisma.crypto.update({
            where: {
                id: crypto.id,
            },
            data: {
                value: newValue,
            },
        });
        const prevAmount = buyer.dollarAvailables;
        await this.prisma.user.update({
            where: {
                id: userId,
            },
            data: {
                dollarAvailables: prevAmount - price,
            },
        });
        const newBalanceSeller = seller.dollarAvailables + price;
        await this.prisma.user.update({
            where: {
                id: seller.id,
            },
            data: {
                dollarAvailables: newBalanceSeller,
            },
        });
        await this.prisma.offer.delete({
            where: {
                id: offer.id,
            },
        });
        return trade;
    }
};
exports.TradeService = TradeService;
exports.TradeService = TradeService = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [prisma_service_1.PrismaService])
], TradeService);
//# sourceMappingURL=trade.service.js.map